@@ -194,7 +194,7 @@ class Ecobee(object):
         params = {"json": json.dumps(param_string)}
         log_msg_action = "get thermostats"
 
-        response = self._request(
+        response = self._request_with_refresh(
             "GET", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, params=params
         )
 
@@ -232,7 +232,9 @@ class Ecobee(object):
         log_msg_action = "set HVAC mode"
 
         try:
-            self._request("POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body)
+            self._request_with_refresh(
+                "POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body
+            )
         except (ExpiredTokenError, InvalidTokenError) as err:
             raise err
 
@@ -248,7 +250,9 @@ class Ecobee(object):
         log_msg_action = "set fan minimum on time"
 
         try:
-            self._request("POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body)
+            self._request_with_refresh(
+                "POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body
+            )
         except (ExpiredTokenError, InvalidTokenError) as err:
             raise err
 
@@ -295,15 +299,17 @@ class Ecobee(object):
         log_msg_action = "set fan mode"
 
         try:
-            self._request("POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body)
+            self._request_with_refresh(
+                "POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body
+            )
         except (ExpiredTokenError, InvalidTokenError) as err:
             raise err
 
     def set_hold_temp(
         self,
         index: int,
-        cool_temp: int,
-        heat_temp: int,
+        cool_temp: float,
+        heat_temp: float,
         hold_type: str = "nextTransition",
         hold_hours: str = "2",
     ) -> None:
@@ -346,7 +352,9 @@ class Ecobee(object):
         log_msg_action = "set hold temp"
 
         try:
-            self._request("POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body)
+            self._request_with_refresh(
+                "POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body
+            )
         except (ExpiredTokenError, InvalidTokenError) as err:
             raise err
 
@@ -373,7 +381,9 @@ class Ecobee(object):
         log_msg_action = "set climate hold"
 
         try:
-            self._request("POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body)
+            self._request_with_refresh(
+                "POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body
+            )
         except (ExpiredTokenError, InvalidTokenError) as err:
             raise err
 
@@ -381,8 +391,8 @@ class Ecobee(object):
         self,
         index: int,
         vacation_name: str,
-        cool_temp: int,
-        heat_temp: int,
+        cool_temp: float,
+        heat_temp: float,
         start_date: str = None,
         start_time: str = None,
         end_date: str = None,
@@ -416,7 +426,9 @@ class Ecobee(object):
         log_msg_action = "create a vacation"
 
         try:
-            self._request("POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body)
+            self._request_with_refresh(
+                "POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body
+            )
         except (ExpiredTokenError, InvalidTokenError) as err:
             raise err
 
@@ -432,7 +444,9 @@ class Ecobee(object):
         log_msg_action = "delete a vacation"
 
         try:
-            self._request("POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body)
+            self._request_with_refresh(
+                "POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body
+            )
         except (ExpiredTokenError, InvalidTokenError) as err:
             raise err
 
@@ -450,7 +464,9 @@ class Ecobee(object):
         log_msg_action = "resume program"
 
         try:
-            self._request("POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body)
+            self._request_with_refresh(
+                "POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body
+            )
         except (ExpiredTokenError, InvalidTokenError) as err:
             raise err
 
@@ -469,7 +485,9 @@ class Ecobee(object):
         log_msg_action = "send message"
 
         try:
-            self._request("POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body)
+            self._request_with_refresh(
+                "POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body
+            )
         except (ExpiredTokenError, InvalidTokenError) as err:
             raise err
 
@@ -485,7 +503,9 @@ class Ecobee(object):
         log_msg_action = "set humidifier mode"
 
         try:
-            self._request("POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body)
+            self._request_with_refresh(
+                "POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body
+            )
         except (ExpiredTokenError, InvalidTokenError) as err:
             raise err
 
@@ -501,7 +521,9 @@ class Ecobee(object):
         log_msg_action = "set humidity level"
 
         try:
-            self._request("POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body)
+            self._request_with_refresh(
+                "POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body
+            )
         except (ExpiredTokenError, InvalidTokenError) as err:
             raise err
 
@@ -517,7 +539,9 @@ class Ecobee(object):
         log_msg_action = "set mic mode"
 
         try:
-            self._request("POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body)
+            self._request_with_refresh(
+                "POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body
+            )
         except (ExpiredTokenError, InvalidTokenError) as err:
             raise err
 
@@ -537,7 +561,9 @@ class Ecobee(object):
         log_msg_action = "set occupancy modes"
 
         try:
-            self._request("POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body)
+            self._request_with_refresh(
+                "POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body
+            )
         except (ExpiredTokenError, InvalidTokenError) as err:
             raise err
 
@@ -553,10 +579,140 @@ class Ecobee(object):
         log_msg_action = "set dst mode"
 
         try:
-            self._request("POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body)
+            self._request_with_refresh(
+                "POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body
+            )
         except (ExpiredTokenError, InvalidTokenError) as err:
             raise err
 
+    def set_vent_mode(self, index: int, vent_mode: str) -> None:
+        """Sets the ventilator mode. Values: auto, minontime, on, off."""
+        body = {
+            "selection": {
+                "selectionType": "thermostats",
+                "selectionMatch": self.thermostats[index]["identifier"],
+            },
+            "thermostat": {"settings": {"vent": vent_mode}},
+        }
+        log_msg_action = "set vent mode"
+
+        try:
+            self._request_with_refresh(
+                "POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body
+            )
+        except (ExpiredTokenError, InvalidTokenError) as err:
+            raise err
+
+    def set_ventilator_min_on_time(self, index: int, ventilator_min_on_time: int) -> None:
+        """Sets the minimum time in minutes the ventilator is configured to run."""
+        body = {
+            "selection": {
+                "selectionType": "thermostats",
+                "selectionMatch": self.thermostats[index]["identifier"],
+            },
+            "thermostat": {"settings": {"ventilatorMinOnTime": ventilator_min_on_time}},
+        }
+        log_msg_action = "set ventilator minimum on time"
+
+        try:
+            self._request_with_refresh(
+                "POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body
+            )
+        except (ExpiredTokenError, InvalidTokenError) as err:
+            raise err
+
+    def set_ventilator_min_on_time_home(self, index: int, ventilator_min_on_time_home: int) -> None:
+        """Sets the number of minutes to run ventilator per hour when home."""
+        body = {
+            "selection": {
+                "selectionType": "thermostats",
+                "selectionMatch": self.thermostats[index]["identifier"],
+            },
+            "thermostat": {"settings": {"ventilatorMinOnTimeHome": ventilator_min_on_time_home}},
+        }
+        log_msg_action = "set ventilator minimum on time when homw"
+
+        try:
+            self._request_with_refresh(
+                "POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body
+            )
+        except (ExpiredTokenError, InvalidTokenError) as err:
+            raise err
+
+    def set_ventilator_min_on_time_away(self, index: int, ventilator_min_on_time_away: int) -> None:
+        """Sets the number of minutes to run ventilator per hour when away."""
+        body = {
+            "selection": {
+                "selectionType": "thermostats",
+                "selectionMatch": self.thermostats[index]["identifier"],
+            },
+            "thermostat": {"settings": {"ventilatorMinOnTimeAway": ventilator_min_on_time_away}},
+        }
+        log_msg_action = "set ventilator minimum on time when away"
+
+        try:
+            self._request_with_refresh(
+                "POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body
+            )
+        except (ExpiredTokenError, InvalidTokenError) as err:
+            raise err
+
+    def set_ventilator_timer(self, index: int, ventilator_on: bool) -> None:
+        """Sets whether the ventilator timer is on or off."""
+        body = {
+            "selection": {
+                "selectionType": "thermostats",
+                "selectionMatch": self.thermostats[index]["identifier"],
+            },
+            "thermostat": {"settings": {"isVentilatorTimerOn": ventilator_on}},
+        }
+        log_msg_action = "set ventilator timer"
+
+        try:
+            self._request_with_refresh(
+                "POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body
+            )
+        except (ExpiredTokenError, InvalidTokenError) as err:
+            raise err
+
+    def _request_with_refresh(
+        self,
+        method: str,
+        endpoint: str,
+        log_msg_action: str,
+        params: dict = None,
+        body: dict = None,
+        auth_request: bool = False,
+    ) -> Optional[str]:
+        """
+        Wrapper around _request, to refresh tokens if needed.
+        If an ExpiredTokenError is seen call refresh_tokens and
+        try one more time. Otherwise, send the results up
+        """
+        response = None
+        refreshed = False
+        for _ in range(0, 2):
+            try:
+                response = self._request(
+                    method, endpoint, log_msg_action, params, body, auth_request
+                )
+            except ExpiredTokenError:
+                if not refreshed:
+                    # Refresh tokens and try again
+                    self.refresh_tokens()
+                    refreshed = True
+                    continue
+                else:
+                    # Send the exception up the stack otherwise
+                    raise
+            except InvalidTokenError:
+                raise
+
+            # Success, fall out of the loop
+            break
+
+        return response
+
     def _request(
         self,
         method: str,
